/*
 * jobs.h
 *
 * Recording information about the print jobs
 * Copyright (c) 1988, 89, 90, 91, 92, 93 Miguel Santana
 * Copyright (c) 1995, 96, 97 Akim Demaille, Miguel Santana
 *
 */

/*
 * This file is part of a2ps.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * $Id: jobs.h,v 1.71 1997/11/07 09:56:15 demaille Exp $
 */
#ifndef _JOBS_H_
#define _JOBS_H_

#include "encoding.h"
#include "output.h"
#include "confg.h"
#include "media.h"
#include "darray.h"
#include "gen.h"
#include "psstat.h"
#include "caret.h"

/* Should the virtual page go first to the left, or the bottom? */
typedef enum {
  major_rows, major_columns
} MAJOR;

struct stream;

/*
 * Some information about the user
 */
struct user_info {
  char * home;
  char * pw_gecos;
  char * pw_name;
};

/* 
 * Relative to the current file
 */
struct file_job {
  uchar * name;			/* Guess what it is :)			*/
  char * tmp_filename;		/* A temporary filename associated to
				 * this file				*/
  const char * type;		/* i.e. ssh key, or UNPRINTABLE		*/
  int is_toc;			/* Is this a toc, or a real file (used
				 * when --pages=toc)			*/
  
  struct tm mod_tm;		/* Modif. info for the file 		*/
  int printable;		/* Can this file be printed?		*/
  int first_sheet;		/* num of the first sheet for this file	*/
  int last_sheet;
  int first_page;		/* Id. but pages			*/
  int last_page;
  int pages;			/* These are defined because it eases	*/
  int sheets;			/* delayed integers (cf. output_marker)	*/
  int num;			/* No of the file in the args		*/
  int top_line;			/* The top most line of the current page */
  int top_page;			/* The first page appearing in curr sheet */
  int lines;			/* Current line number			*/
};

/* All the information on the current print job */
typedef struct a2ps_job {
  char ** argv;			/* The command line			*/
  int argc;

  /* Path to the files used by a2ps	*/
  char ** lib_path;
  char * tmpdir;		/* Temporary dir name			*/
  char * file_command;		/* Command to use to call file(1)	*/

  struct user_info * user;	/* User information			*/

  /* Global information that the library needs */
  hash_table * media;		/* Definition of the media		*/
  struct pair_htable * user_options; /* Short cuts defined by the user	*/

  /* Relative to the whole process */
  struct tm run_tm; 		/* Time when this program is run	*/
  char datestring[256];		/* Full today's date			*/
  int sheets;			/* number of sheets before the curr file*/
  int pages;			/* id. but pages			*/
  int total_files;		/* id but files				*/
  ORIENTATION orientation;	/* landscape mode ?			*/
  int rectoverso;		/* Recto verso printing ? 		*/
  int columns;			/* number of columns/page		*/
  int rows;			/* number of rows/page			*/
  MAJOR Major;			/* firt rows, or columns?
				 * (not major, because of a SunOS macro)*/
  int virtual;			/* number of the virtual page on that sheet */
  int copies;			/* number of copies			*/
  int margin;			/* margin to leave for binding		*/

  struct pair_htable * encodings_map;/* Content of the encoding.map file*/
  struct output * ps_encodings; /* PS definition of the encodings used	*/
  int page_prefeed;		/* Page prefeed mecanism		*/
  /* How to get the physical font (eg Courier), from the logical
   * (eg, Keyword) */
  char * face_to_font [NB_FACES];
  uchar * stdin_filename;	/* Filename given to stdin		*/

  /*
   * Output related
   */
  enum output_format output_format; /* ps, eps etc.			*/
  struct pair_htable * printers;/* The user defined available printers	*/
  struct pair_htable * printers_map;	/* printer name -> pdd		*/
  struct ppd * ppd;		/* Name of the ppd file asked		*/
  int lpr_print; 		/* output_name is about printer?	*/
  				/* Otherwise, it's a file name		*/
  char *output_name;		/* if lpr_print, NULL -> default printer
				   if not, NULL-> stdout		*/
  struct stream * output_stream;/* Where the result will be dumped	*/

  int folding;			/* Line folding option 			*/
  int numbering;		/* Line numbering option 		*/
  /* Replace non printable char */
  enum unprintable_format unprintable_format;
  int interpret;		/* Interpret TAB, FF and BS chars option */
  int print_binaries;		/* Force printing for binary files 	*/
  int compact_mode;		/* Allow 2 files per sheet 		*/
  int border;			/* print the surrounding border ?	*/
  int debug;			/* Are we in debug mode?		*/
  char * prolog;		/* postscript header file 		*/
  struct medium * medium;	/* Medium to use 			*/
  int tabsize;			/* length of tabulations		*/
  int lines_requested;		/* Number of line per page 		*/
  int columns_requested;	/* Number of columns per page		*/
  float fontsize;		/* Size of a char for body font 	*/
  struct encoding_entry * encoding;/* What is the current char set ?	*/
  char * requested_encoding_name;/* Because encoding.map has not been
				  * read when reading the conf files,
				  * we cannot store the result of -X<enc>
				  * as an encoding_entry */
  struct encoding_entry * requested_encoding; 	/* The main encoding (given by option)	*/
  struct encoding_entry * saved_encoding; 	/* Given by the options			*/
  struct hash_table * encodings;/* The hash table of encodings yet read */

  struct pair_htable * fonts_map;/* Aliases (to cope with MS_DOS) */
  struct hash_table * font_infos;	/* The HT of the afm for each file */

  /* Headers and footers */
  uchar * title;		/* Job's title (eg. in %%Title: )	*/
  uchar * header;		/* Allow different header text	 	*/
  uchar * center_title;		/* Allow different header text	 	*/
  uchar * left_title;		/* Allow different header text	 	*/
  uchar * right_title;		/* Allow different header text	 	*/
  uchar * left_footer;		/* Allow different header text	 	*/
  uchar * footer;		/* Allow different header text	 	*/
  uchar * right_footer;		/* Allow different footer text	 	*/
  uchar * water;		/* Water marks text			*/

  /* Used to grab headers etc. from the file */
  uchar tag1[256], tag2[256], tag3[256], tag4[256];

  /* Definition of the macro meta sequences	*/
  struct pair_htable * macro_meta_sequences;

  /* Private information on the PS generation engine */
  struct ps_status * status;

  /* Where the output is diverted */
  struct output * divertion;

  /* 10 temporary files */
  char * tmp_filenames [10];
  
  /* Interval of pages to print */
  struct page_range * page_range;

  /* List of the jobs */
  struct darray * jobs;
} a2ps_job;

/* liba2ps.h:begin */
/* Return a newly allocated output session storage */
struct a2ps_job * a2ps_job_new PARAMS ((void));

/* Finalize it */
void a2ps_job_finalize PARAMS ((struct a2ps_job * job));

/* In the output session JOB, create a new input session NAME */
void a2ps_open_input_session PARAMS ((struct a2ps_job * job, uchar * name));

/* Free the memory used by JOB */
void a2ps_job_free PARAMS ((struct a2ps_job * job));
/* liba2ps.h:end */

int file_name_cmp PARAMS ((struct file_job * f1, struct file_job * f2));

void file_job_synchronize_sheets PARAMS ((struct a2ps_job * job));
void file_job_synchronize_pages PARAMS ((struct a2ps_job * job));

#define CURRENT_FILE(j)				\
  ((struct file_job *) (j->jobs->content[j->jobs->len - 1]))
#define FIRST_FILE(j)				\
  ((struct file_job *) (j->jobs->content[0]))

/* Unlink everything that could have been used */
void a2ps_job_unlink_tmpfiles PARAMS ((struct a2ps_job * job));

#endif
