/*
 * ppd.c
 *
 * PostScript Printer Description files
 *
 * Copyright (c) 1988, 89, 90, 91, 92, 93 Miguel Santana
 * Copyright (c) 1995, 96, 97 Akim Demaille, Miguel Santana
 */

/*
 * This file is part of a2ps.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * $Id: dsc.c,v 1.3 1997/09/23 16:05:53 demaille Exp $
 */

#include "a2ps.h"
#include "ppd.h"
#include "jobs.h"
#include "routines.h"
#include "pathwalk.h"
#include "pair_ht.h"

/* From parsepdd.y */
struct ppd * a2ps_ppd_parse PARAMS ((const char * filename));

/************************************************************************/
/*	Handling the fonts						*/
/************************************************************************/
/*
 * Idealy, this should be done through pseudo PPD files
 */
int
font_is_ps_known (struct a2ps_job * job, const char * name)
{
  int i;
  static const char * known_fonts [] = {
    
    "Courier", "Courier-Bold", "Courier-Italic", "Courier-Oblique",
    "Courier-BoldItalic", "Courier-BoldOblique",
    
    "Times-Roman", "Times-Bold", "Times-Italic", "Times-Oblique",
    "Times-BoldItalic", "Times-BoldOblique",
    
    "Helvetica", "Helvetica-Bold", "Helvetica-Italic",
    "Helvetica-Oblique", "Helvetica-BoldItalic", "Helvetica-BoldOblique",
    
    "Symbol",
    
    NULL
  };
  
  for (i = 0 ; known_fonts [i] ; i++)
    if (strequ (known_fonts [i], name))
      return TRUE;
  return FALSE;
}

void 
ppd_font_add (struct ppd * ppd, const char * fontname)
{
  pair_add (ppd->fonts, fontname, NULL);
}

int
ppd_font_known_p (struct ppd * ppd, const char * fontname)
{
  return (pair_get (ppd->fonts, fontname) != NULL);
}

/************************************************************************/
/*	Handling the PPD structure					*/
/************************************************************************/
struct ppd *
ppd_new (void)
{
  NEW (struct ppd, res);
  res->fonts = pair_table_new ();
  return res;
}

void
ppd_free (struct ppd * ppd)
{
  pair_table_free (ppd->fonts);
  FREE (ppd);
}

void
ppd_self_print (struct ppd * ppd, FILE * stream)
{
  fputs ("List of the known fonts:\n", stream);
  pair_table_self_print (ppd->fonts, stream);
}

/************************************************************************/
/*	Handling the printers.map file					*/
/************************************************************************/
struct pair_htable *
printers_map_new (void)
{
  return pair_table_new ();
}

void
printers_map_free (struct pair_htable * table)
{
  pair_table_free (table);
}

static const char *
printers_map_ppd_name_get (struct a2ps_job * job,
			   const char * printer_name)
{
  const char * res;

  if (job->lpr_print) 
    {
      /* Is a printer */
      if (!IS_EMPTY (printer_name))
	{
	  /* Is a named printer */
	  res = pair_get (job->printers_map, printer_name);
	  if (!res)
	    res = pair_get (job->printers_map, "UnknownPrinter:");
	}
      else
	{
	  /* Is the default printer */
	  res = pair_get (job->printers_map, "DefaultPrinter:");
	}
    }
  else
    {
      /* Is a file */
      res = pair_get (job->printers_map, "Default:");
    }
  return res;
}

struct ppd *
printers_map_ppd_get (struct a2ps_job * job,
		      const char * printer_name)
{
  struct ppd * res;
  const char * ppd_filename;
  char * ppd_filepath;

  ppd_filename = printers_map_ppd_name_get (job, printer_name);
  ppd_filepath = xpw_find_file (job->lib_path, ppd_filename, ".ppd");
  res = a2ps_ppd_parse (ppd_filepath);
  FREE (ppd_filepath);
  return res;
}

/*
 * Read a sheets map file
 */
static int
printers_map_load (struct a2ps_job * job, const char *file)
{
  return pair_table_load (job->printers_map, file);
}

/*
 * Read the printers.map file
 */
int
printers_map_main_load (struct a2ps_job * job)
{
  char * file;

  /* System's */
  file = xpw_find_file (job->lib_path, "printers.map", NULL);
  printers_map_load (job, file);
  FREE (file);
  return 1;
}
